/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
	      programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

#include "alphCRYPT.h"

char
*beale(char *org, char *key)
{

  char *pkey, *porg;
  int itra, itrb, itrc;
  int *ktrix[26];
  int count, runner;
  char *ibuf;

  void *empty=0;

  int sem;
  char *dst;

  pkey=(char *) calloc(strlen(key), sizeof(char));
  porg=(char *) calloc(strlen(org), sizeof(char));

  /* Reduce case complexity. */
  for(itra=0, itrb=0; itra<strlen(org) && !al_dec; itra++)
    porg[itrb++]=tolower(org[itra]);

  for(itra=0, itrb=0; itra<strlen(key); itra++)
    if(isalpha(key[itra]))    
      pkey[itrb++]=tolower(key[itra]);

  /* Check key entropy. */
  for(itra=0; itra<strlen(porg) && !al_dec; itra++)
    if(isalpha(porg[itra]) && !strchr(pkey, tolower(porg[itra])))
      fault(ENTO, "The key does not contain some of the letters in the original message.");

  /* Gather keying statistics. */
  for(itra='a'; itra<='z'; itra++)
    {
      for(itrb=0, count=0; itrb<strlen(pkey); itrb++)
        if(itra==pkey[itrb])
          count++;

      /* Initialize key matrix, fuzz 3. */
      ktrix[itra-97]=(int *) calloc(count, sizeof(int));
      for(itrb=0; itrb<count+3; itrb++)
        ktrix[itra-97][itrb]=0;

      for(itrb=0, itrc=0; itrb<strlen(pkey); itrb++)
        if(itra==pkey[itrb])
          ktrix[itra-97][itrc++]=itrb+1;
    }

  /* Do encryption. */
  dst=(char *) calloc(strlen(porg)*10, sizeof(char));
  for(itra=0; itra<strlen(porg) && !al_dec; itra++, srand(time(empty)))
    {
      /* Gather entropy. */
      for(itrc=strlen(porg); itrc>0; itrc--)
        srand(rand()*time(empty)*itrc+itra);

      if(isalpha(porg[itra]))
        {
          /* How much statistic in our key matrix? */
          for(itrb=0, count=0; ktrix[porg[itra]-97][itrb]>0; itrb++)
            count++;
          ibuf=(char *) calloc(count, sizeof(char));
          sprintf(ibuf, "%i", ktrix[porg[itra]-97][rand()%count]);
          strcat(dst, ibuf);
          if(isalpha(porg[itra+1]))
            strcat(dst, "-");
        }
      else
        dst[strlen(dst)]=porg[itra];
    }

  ibuf=(char *) calloc(10, sizeof(char));
  for(itra=0, count=0, sem=0, itrc=0; itra<strlen(org) && al_dec; itra++)
    {
      if(isdigit(org[itra]))
        ibuf[count++]=org[itra];
      else
        sem++;

      if(sem || itra==strlen(org)-1)
        {
          /* Got number. Mack it! */
          if(strlen(ibuf))
            dst[itrc++]=pkey[atoi(ibuf)-1];
          
          ibuf=(char *) calloc(10, sizeof(char));
          sem--;
          count=0;
        }
      if(!isdigit(org[itra]) && org[itra]!='-')
        dst[itrc++]=org[itra];
    }

  return dst;

}
